
import java.util.ArrayList;
import java.util.HashMap;
/**
 * SmartGuess add a little "AI" to guesses of an integer nature
 * It keeps track of the last 4 guess pattern and 
 * stores a history of the moves made for that pattern
 * It then returns a move based on the most popular move 
 * made with that pattern. If there is no history for that
 * pattern, it makes a random choice from 0 to max.
 *  
 * @author Chris Thiel, OFMCap
 * @version Nov 26, 2011
 * @param choices 
 */

public class SmartGuesser 
{
	private HashMap<Integer,String> history;
	private int currentPattern;
	private int guessCount;
	private int maxGuess;
	private ArrayList<GestureButton> choices;
	public SmartGuesser(ArrayList<GestureButton> choices){
		history=new HashMap<Integer,String>();
		currentPattern = 0;
		guessCount=0;
		this.choices=choices;
		this.maxGuess=choices.size()-1;
	}
	public void record(int move){
		if (guessCount>3){
			String value="";
			if (history.containsKey(currentPattern) )
				value=history.get(currentPattern);
			history.put(currentPattern, value+move);
		}
		guessCount++;
		currentPattern=(10*currentPattern+move)%1000;
		
	}
	public int getGuess(){
		if (guessCount>3 && history.containsKey(currentPattern)){
		       int theirChoice=popularChoice(history.get(currentPattern));
		       int guess=0;
		       while(choices.get(guess).beats(choices.get(theirChoice))<1){
		    	   		guess++;
		       }
		    	   return guess;
		}
		return (int)((maxGuess+1)*Math.random());
	}
	public int popularChoice(String h){
		int [] count=new int[maxGuess+1];
		for (int i=0; i<h.length(); i++)
			count[Integer.parseInt(h.substring(i, i+1))]++;
		int max=0;
		for (int i=0; i<count.length; i++)
			if (count[i]>count[max])
				max=i;
		return max;
	}
}
